// Two Color Twinkle
// 9/3/18 - 12/13/19 Carl F Sutter

#include <Adafruit_NeoPixel.h>
#ifdef __AVR__
  #include <avr/power.h>
#endif

// Neopixel constants
#ifdef ESP8266
  #define PIN D6       // ESP8266 pin for the LED strand data line - GPIO 12
  #define NUMLEDS 338  // the # of LEDs on Carl's house 
#else
  #define PIN 6        // Arduino pin for the LED strand data line
  #define NUMLEDS 200  // 200 is about the max an Arduino Uno or Nano can handle
#endif // ESP8266

// App constants
// The RGB values for the first color 0-255
#define COLOR1_R 255
#define COLOR1_G 0
#define COLOR1_B 0

// The RGB values for the second color 0-255
#define COLOR2_R 0
#define COLOR2_G 255
#define COLOR2_B 0

// loop control - these are the main constants to control the animation
// note that the math is in integers right now, so if the step size is greater than the color values, it wil not work well
#define NUM_STEPS_UP    5    // the number of steps to fade the color up
#define NUM_STEPS_DOWN  30   // the number of stesp to fade the color down
#define LOOP_DELAY      10   // the ms delay for each loop
#define BLINK_ODDS      5    // the odds in 1000 that the loop will start each black pixel

// internal constants and vars - no need to change
#define STATE_BLACK          0
#define STATE_MOVING_UP_1    1
#define STATE_MOVING_DOWN_1  2
#define STATE_MOVING_UP_2    3
#define STATE_MOVING_DOWN_2  4

uint8_t state[NUMLEDS]; 
uint8_t color_r[NUMLEDS]; 
uint8_t color_g[NUMLEDS]; 
uint8_t color_b[NUMLEDS]; 


// Parameter 1 = number of pixels in strip
// Parameter 2 = Arduino pin number (most are valid)
// Parameter 3 = pixel type flags, add together as needed:
//   NEO_KHZ800  800 KHz bitstream (most NeoPixel products w/WS2812 LEDs)
//   NEO_KHZ400  400 KHz (classic 'v1' (not v2) FLORA pixels, WS2811 drivers)
//   NEO_GRB     Pixels are wired for GRB bitstream (most NeoPixel products)
//   NEO_RGB     Pixels are wired for RGB bitstream (v1 FLORA pixels, not v2)
//   NEO_RGBW    Pixels are wired for RGBW bitstream (NeoPixel RGBW products)
Adafruit_NeoPixel strip = Adafruit_NeoPixel(NUMLEDS, PIN, NEO_GRB + NEO_KHZ800);

// IMPORTANT: To reduce NeoPixel burnout risk, add 1000 uF capacitor across
// pixel power leads, add 300 - 500 Ohm resistor on first pixel's data input
// and minimize distance between Arduino and first pixel.  Avoid connecting
// on a live circuit...if you must, connect GND first.


void setup() {
  // set all the LEDS to the no blink (black) state
  for(int i=0; i<NUMLEDS; i++) {
    state[i] = STATE_BLACK;
    color_r[i] = 0;
    color_g[i] = 0;
    color_b[i] = 0;
  }

  randomSeed(analogRead(0));

  strip.begin();
  strip.show(); // Initialize all pixels to 'off'
} // setup


void loop() {
  for (int i=0; i<NUMLEDS; i++) {
    switch (state[i]) {
      case STATE_BLACK:
        strip.setPixelColor(i,  0x000000);
        if (random(1000) < BLINK_ODDS) {
          if (random(2) == 0) {
            state[i] = STATE_MOVING_UP_1;
          } else {
            state[i] = STATE_MOVING_UP_2;
          }
        } // BLINK_ODDS
        break;

      case STATE_MOVING_UP_1:
         color_r[i] = min(COLOR1_R, color_r[i] + (COLOR1_R / NUM_STEPS_UP));
         color_g[i] = min(COLOR1_G, color_g[i] + (COLOR1_G / NUM_STEPS_UP));
         color_b[i] = min(COLOR1_B, color_b[i] + (COLOR1_B / NUM_STEPS_UP));
         strip.setPixelColor(i, color_r[i], color_g[i], color_b[i]);
         if ((color_r[i] == COLOR1_R) && (color_g[i] == COLOR1_G) && (color_b[i] == COLOR1_B)) {
           state[i] = STATE_MOVING_DOWN_1;
         }
         // maybe someday make the code cleaner and use less memory - OK for now
         //uint32_t ColorOfThePixel = MyPixels.getPixelColor(12);
         //uint8_t r = ColorOfThePixel >> 16;
         //uint8_t g = ColorOfThePixel >> 8;
         //uint8_t b = ColorOfThePixel;
        break;

      case STATE_MOVING_UP_2:
         color_r[i] = min(COLOR2_R, color_r[i] + (COLOR2_R / NUM_STEPS_UP));
         color_g[i] = min(COLOR2_G, color_g[i] + (COLOR2_G / NUM_STEPS_UP));
         color_b[i] = min(COLOR2_B, color_b[i] + (COLOR2_B / NUM_STEPS_UP));
         strip.setPixelColor(i, color_r[i], color_g[i], color_b[i]);
         if ((color_r[i] == COLOR2_R) && (color_g[i] == COLOR2_G) && (color_b[i] == COLOR2_B)) {
           state[i] = STATE_MOVING_DOWN_2;
         }
        break;
        
      case STATE_MOVING_DOWN_1:
         color_r[i] = max(0, color_r[i] - max(1, (COLOR1_R / NUM_STEPS_DOWN)));
         color_g[i] = max(0, color_g[i] - max(1, (COLOR1_G / NUM_STEPS_DOWN)));
         color_b[i] = max(0, color_b[i] - max(1, (COLOR1_B / NUM_STEPS_DOWN)));
         strip.setPixelColor(i, color_r[i], color_g[i], color_b[i]);
         if ((color_r[i] == 0) && (color_g[i] == 0) && (color_b[i] == 0)) {
           state[i] = STATE_BLACK;
         }
        break;
        
      case STATE_MOVING_DOWN_2:
         color_r[i] = max(0, color_r[i] - max(1, (COLOR2_R / NUM_STEPS_DOWN)));
         color_g[i] = max(0, color_g[i] - max(1, (COLOR2_G / NUM_STEPS_DOWN)));
         color_b[i] = max(0, color_b[i] - max(1, (COLOR2_B / NUM_STEPS_DOWN)));
         strip.setPixelColor(i, color_r[i], color_g[i], color_b[i]);
         if ((color_r[i] == 0) && (color_g[i] == 0) && (color_b[i] == 0)) {
           state[i] = STATE_BLACK;
         }
        break;
        
        } // switch
    } // led loop
    
    strip.show();
    delay(LOOP_DELAY);
 } // loop
